/*******************************************************************************
*    MultiButton 1.0.                                                          *
*    Verwaltung eines Domotikasystem mit O2-Joggler                            *
*    nach einer Idee von Leonardo Buffa <leonardo.buffa@gmail.com>             *
*    Copyright (C) 2013-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <wx/cmdline.h>
#include <wx/stdpaths.h>
#include <wx/filename.h>
#include <wx/fileconf.h>
#include <wx/tokenzr.h>
#include <wx/dir.h>
#include <wx/utils.h>
#include "MainFrame.h"
#include "lib/XML.h"

/**
 * Hauptklasse des Programms
 */
class MultiButton: public wxApp
{
  /**
   * Initialisierung
   */
  virtual bool OnInit();

private:
  MainFrame *mainFrame;
};

IMPLEMENT_APP(MultiButton)

/**
 * Initialisierung
 */
bool MultiButton::OnInit()
{
  wxString confFile, resolution, pageXML, baseURL, imageBaseURL, cacheDir, beepDevice;
  int width, height;
  bool error = true;
  XML firstPage;
  FILE *fp;

  this->mainFrame = NULL;

  static const wxCmdLineEntryDesc cmdLineDesc [] =
  {
       { wxCMD_LINE_SWITCH, wxT("h"), wxT("help"), wxT("show this help message"),
            wxCMD_LINE_VAL_NONE, wxCMD_LINE_OPTION_HELP },
       { wxCMD_LINE_SWITCH, wxT("v"), wxT("version"), wxT("display the program version and exit"),
            wxCMD_LINE_VAL_NONE, wxCMD_LINE_SWITCH },
       { wxCMD_LINE_SWITCH, wxT("x"), wxT("clearcache"), wxT("clear the image cache, specified in the config file"),
            wxCMD_LINE_VAL_NONE, wxCMD_LINE_SWITCH },
       { wxCMD_LINE_OPTION, wxT("c"), wxT("config"), wxT("use the given config file"),
            wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL  },

       { wxCMD_LINE_NONE }
  };
  wxCmdLineParser parser(cmdLineDesc, wxApp::argc, wxApp::argv);
  if(parser.Parse() == 0)
  {
    if(parser.Found(wxT("version")))
    {
      printf("Multibutton version %s\n", VERSION);
      printf("Authors:\n\tLuca Bertoncello <lucabert@lucabert.de>\n");
      printf("\tLeonardo Buffa <leonardo.buffa@gmail.com>\n\n");
      printf("This program is available under the GPL-2.0 license\n");
      printf("and WITHOUT ANY WARRANTY.\n");
      exit(0);
    }
    if(!parser.Found(wxT("config"), &confFile))
    {
      wxFileName fn(wxStandardPaths::Get().GetExecutablePath());
      confFile = fn.GetPath() + wxFileName::GetPathSeparator() + wxT("multibutton.cfg");
    }

    wxFileInputStream stream(confFile);
    wxFileConfig config(stream);
    config.SetPath(wxT("multibutton"));
    config.Read(wxT("resolution"), &resolution);
    config.Read(wxT("baseURL"), &baseURL);
    config.Read(wxT("imagesDir"), &imageBaseURL);
    config.Read(wxT("firstpage"), &pageXML);
    config.Read(wxT("cacheDir"), &cacheDir);
    config.Read(wxT("beepDevice"), &beepDevice);
    if(beepDevice.Length() != 0)
    {
      fp = fopen((const char*) beepDevice.mb_str(), "r");
      if(fp != NULL)
        fclose(fp);
      else
      {
        printf("Device %s does not exists. Disabling beep...\n", (const char*) beepDevice.mb_str());
        beepDevice = wxT("");
      }
    }
    else
      beepDevice = wxT("/dev/dsp");
    if(!cacheDir.EndsWith(wxT("/")))
      cacheDir = cacheDir + wxT("/");
    if(!baseURL.EndsWith(wxT("/")))
      baseURL = baseURL + wxT("/");
    if(imageBaseURL.Length() == 0)
      imageBaseURL = baseURL;
    if(!imageBaseURL.EndsWith(wxT("/")))
      imageBaseURL = imageBaseURL + wxT("/");
    if(!pageXML.StartsWith(wxT("http://")) &&
       !pageXML.StartsWith(wxT("https://")) &&
       !pageXML.StartsWith(wxT("file://")))
      pageXML = baseURL + pageXML;
    if(!imageBaseURL.StartsWith(wxT("http://")) &&
       !imageBaseURL.StartsWith(wxT("https://")) &&
       !imageBaseURL.StartsWith(wxT("file://")))
      imageBaseURL = baseURL + imageBaseURL;
    if(resolution.Length() != 0 && pageXML.Length() != 0)
    {
      wxStringTokenizer st(resolution, wxT("x"));
      if(st.CountTokens() == 2)
      {
        width = wxAtoi(st.GetNextToken());
        height = wxAtoi(st.GetNextToken());
        if(width > 1 && height > 1)
        {
          if(parser.Found(wxT("clearcache"), &confFile))
          {
            if(cacheDir.Length() != 0)
            {
              printf("Clearing image cache...\n");
              wxDir dir(cacheDir);

              if(dir.IsOpened() )
              {
                wxString filename;
                bool cont = dir.GetFirst(&filename);
                while(cont)
                {
                  wxRemoveFile(cacheDir + filename);
                  cont = dir.GetNext(&filename);
                }
              }
            }
            else
              printf("No cache directory configured. Ignoring...\n");
          }
          if(cacheDir.Length() != 0)
          {
            if(!wxDirExists(cacheDir))
            {
              printf("Cache directory %s does not exists. Creating it...\n", (const char*) cacheDir.mb_str());
              if(!wxMkdir(cacheDir))
              {
                printf("Unable to create cache directory. Cache will be deactivated.\n");
                cacheDir = wxT("");
              }
            }
          }
          if(firstPage.loadFile(pageXML, baseURL, imageBaseURL))
          {
            error = false;
            this->mainFrame = new MainFrame(NULL, width, height, firstPage, baseURL, imageBaseURL, cacheDir, beepDevice);
            mainFrame->Show(true);
            SetTopWindow(mainFrame);
          }
          else
          {
            printf("Unable to read XML\n");
            return false;
          }
        }
      }
    }
    if(error)
    {
      printf("The configuration file MUST be like the following:\n");
      printf("[multibutton]\nresolution=800x480\nfirstpage=http://my.server.tld/page1.xml\n");
      return false;
    }
    return true;
  }
  else
    return false;

  return true;
}
